import inspect
import os


class Service:
    unit = None
    _template = '%s@%s.service'

    @staticmethod
    def get_dir(location):
        """
        Returns parent directory's path.
        :rtype: str
        """
        return os.path.dirname(os.path.abspath(location))

    def __init__(self, unit, **kwargs):
        """
        :type unit: str Service name
        """
        self.unit = unit
        if 'autoregister' in kwargs and kwargs['autoregister'] is True:
            storage = ServiceStorage()
            storage.add(self)

    def __str__(self):
        return self.unit

    def template_file(self):
        """
        Returns template filename based on unit name.
        :rtype: str
        """
        return self.instance_file('')

    def instance_file(self, instance_id):
        """
        Returns instance filename based on template filename and instance id.
        :type instance_id: str
        :rtype: str
        """
        return self._template % (self, instance_id)

    def content(self):
        """
        Returns rendered content from template file.
        :rtype: str
        """
        path = self.get_dir(inspect.getfile(self.__class__))
        return open('%s/%s' % (path, 'template'), 'r').read().format(self=self)


class ServiceStorage(object):
    __instance = None
    __services = {}

    def __new__(cls):
        if cls.__instance is None:
            cls.__instance = object.__new__(cls)
        return cls.__instance

    def add(self, service, key=None):
        """
        Adds the Service to storage.
        :type service: Service
        :type key: str if you don't want to use Service.unit as access key to Service in Storage
        :rtype: self
        """
        if isinstance(service, Service):
            if key is None:
                key = service.unit
            self.__services[key] = service
        else:
            raise ValueError('Invalid Service.')
        return self

    def get(self, service_name):
        """
        Returns the Service from storage or None if it is not found.
        :type service_name: str
        :rtype: Service|None
        """
        if service_name in self.__services:
            return self.__services.get(service_name)
        else:
            return None
