from fabric.api import run, settings, sudo, task
from fabric.contrib.console import confirm
from fabric.contrib.files import sed
import re


@task()
def restart():
    with settings(abort_on_prompts=False, ok_ret_codes=[-1]):
        if confirm('Do you want to restart the server?'):
            sudo('shutdown -r now')


@task()
def shutdown():
    with settings(abort_on_prompts=False, ok_ret_codes=[-1]):
        if confirm('Do you want to shutdown the server?'):
            sudo('shutdown -h now')


@task()
def switch_channel(channel):
    """
    Run for debug `journalctl -f -u update-engine`.
    :type channel: str switching channel
    """
    available = ['alpha', 'beta', 'stable']
    if channel not in available:
        raise ValueError('Invalid channel "%s".' % channel)
    info = get_version()
    if info.get('channel') == channel:
        print 'Channels are identical.'
    else:
        sed(info.get('filename'), info.get('channel'), channel, backup='', use_sudo=True)
        sudo('systemctl restart update-engine')


def get_version():
    with settings(warn_only=True):
        update_config = '/etc/coreos/update.conf'
        if run('test -e %s' % update_config).failed:
            sudo('cp /usr/share/coreos/update.conf %s' % update_config)
    version = {
        'filename': update_config,
        'channel': re.search(r'(?<==)\w+', run('cat %s | grep GROUP=' % update_config)).group(),
        'release': re.search(r'(?<==)(?:\d+\.?)+', run('cat /etc/os-release | grep VERSION=')).group(),
    }
    return version
