from fabric.api import execute, run, task
from fabric.contrib.files import append
from fabricant.coreos.classes import Service, ServiceStorage


@task()
def publish(service_name, instance_id=''):
    service = resolve(service_name)
    execute(destroy, service, instance_id)
    template = service.template_file()
    instance = service.instance_file(instance_id)
    run('echo > %s' % template)
    append(template, service.content())
    if template != instance:
        run('ln -sf %s %s' % (template, instance))
    load(service, instance_id)
    execute(start, service, instance_id)


@task()
def start(service_name, instance_id=''):
    service = resolve(service_name)
    fleetctl('start', service, instance_id)


@task()
def stop(service_name, instance_id=''):
    service = resolve(service_name)
    fleetctl('stop', service, instance_id)


@task()
def destroy(service_name, instance_id=''):
    service = resolve(service_name)
    execute(stop, service, instance_id)
    fleetctl('destroy', service, instance_id)


def resolve(service):
    """
    :type service: str|Service
    :rtype: Service
    :raise: ValueError
    """
    if isinstance(service, str):
        service_storage = ServiceStorage()
        service = service_storage.get(service)
    if not isinstance(service, Service):
        raise ValueError('Invalid Service passed.')
    return service


def load(service, instance_id):
    """
    :type service: Service
    :type instance_id: str
    """
    fleetctl('load', service, instance_id)


def fleetctl(command, service, instance_id):
    """
    :type command: str
    :type service: Service
    :type instance_id: str
    """
    placeholders = {
        'command': command,
        'service': service.instance_file(instance_id)
    }
    run('fleetctl %(command)s %(service)s' % placeholders)
