from fabric.api import run, settings
from fabricant.tools.classes import Repository


def init(repo):
    """
    Initializes the repository.
    :type repo: Repository|dict
    """
    with settings(warn_only=True):
        if run('test -d %(path)s' % repo).failed:
            run('git clone %(url)s %(path)s --origin %(remote)s' % repo)
        for remote in repo.remotes:
            run('git remote add %(remote)s %(url)s' % remote)


def update(version):
    """
    Updates the repository to the required version.
    :type version: str can be a branch, tag or commit
    """
    clean()
    fetch()
    run('git checkout %s' % version)


def clean():
    """
    Clean the repository.
    """
    run('git reset --hard && git clean -df')


def fetch():
    """
    Download objects and refs from remote repositories.
    """
    run('git fetch --all -p')


def get_remotes():
    """
    Return remote repositories.
    :return:
    """
    return run('git remote -v')


def get_stash():
    """
    Return stash content.
    :return:
    """
    return run('git stash list')


def get_branch():
    """
    Return current branch.
    :return:
    """
    return run('git branch | sed -n "/\* /s///p"')


def get_commit():
    """
    Return current commit.
    :return:
    """
    return run('git rev-parse --short HEAD')


def get_tag():
    """
    Return current tag.
    :return:
    """
    return run('git describe --tags')


def get_tags():
    """
    Return available tags.
    :return:
    """
    return run('git tag list')
